#!/usr/bin/perl

# |-----------------------------------------------------------------------| 
# |                                                                       |
# |    Copyright (C) Avery Design Systems, Inc. 2017                      |
# |    All rights reserved. Licensed software.                            |
# |                                                                       |
# |-----------------------------------------------------------------------|

use Cwd;
use Cwd 'abs_path';
use strict;
use warnings;
use Config;
use File::Basename;

#----------show help-----------------
sub show_usage {
    chomp(my $prog = basename($0));

    my $usage = "
    Script that checks environment and setting issues related to license check   
 
    -h
        Print usuage 

    -l log_file 
        Specify log file 

    example: ./$prog -l license.log
        
    ";

    print("$usage \n");
    exit 0;
}

#----------global varaible-----------
my $avery_dir = "";
my $home = "";
my $lm_license_file = "";
my $log_file = "";
my $ofh;
my $datetime = localtime();

#----------functions-----------------
sub parse_arg {
    my ($argv_ref) = @_;
    my $argc = scalar(@{$argv_ref});

    do { &show_usage if ($_ eq "-h"); } foreach (@{$argv_ref});

    for (my $i = 0; $i < $argc; $i++) {
        my $arg = ${$argv_ref}[$i];
        my $val = ${$argv_ref}[$i+1];

        if ($arg eq "-l"){
            die ("file name not specified\n") if (!defined $val);
            $log_file = $val;
        }
    }
    die ("log file not specified\n") if ($log_file eq "");
}

sub get_sys_info {
    print $ofh ("##### Current time #####\n"); 
    print $ofh ("$datetime\n\n");
    print $ofh ("##### System Information #####\n");
    &get_arch_info;
    &get_os_info;
    &get_gcc_info;
    print $ofh ("\n");
}

sub get_arch_info {
    my $arch = $Config{archname};
   
    print $ofh ("Architecture : "); 
    if (defined $arch) {
        print $ofh ("$arch\n");
    }
    else {
        print $ofh ("Could not detect architecture\n");
    }
}   

sub get_os_info {
    my $msg = "";
    my $os = "";

    $os = `uname -rs`;
    chomp($os);  
 
    print $ofh ("OS : "); 
    if (defined $os) {
        print $ofh ("$os\n");
    }
    else {
        print $ofh ("Could not detect OS\n");
    }
}

sub get_gcc_info {
    my $msg = "";

    $msg = `gcc -v 2>&1 | grep version`;
    chomp($msg);
    print $ofh ("gcc : $msg\n");
}

sub get_lib_info {
    my %check_list = ();
    my $lib = "";
    my $lib32 = "";
    my $lib64 = "";

    $check_list{"LSB"}{"/lib/ld-lsb.so.3"} = "/lib64/ld-lsb-x86-64.so.3";
    $check_list{"pthread"}{"/lib/libpthread.so.0"} = "/lib64/libpthread.so.0";
    $check_list{"zlib"}{"/lib/libz.so.1"} = "/lib64/libz.so.1";
    $check_list{"C++"}{"/lib/libstdc++.so.6"} = "/lib64/libstdc++.so.6";
    $check_list{"math"}{"/lib/libm.so.6"} = "/lib64/libm.so.6";
    $check_list{"gcc run time"}{"/lib/libgcc_s.so.1"} = "/lib64/libgcc_s.so.1";
    $check_list{"interpreter"}{"/lib/ld-linux.so.2"} = "/lib64/ld-linux-x86-64.so.2";
    $check_list{"network service"}{"/lib/libnsl.so.1"} = "/lib64/libnsl.so.1";
    $check_list{"dynamic linking"}{"/lib/libdl.so.2"} = "/lib64/libdl.so.2";

    print $ofh ("##### Required Library Information #####\n"); 
    &get_glibc_info;

    foreach $lib (sort keys %check_list) {
        print $ofh ("$lib : \n");
        foreach $lib32 (sort keys %{$check_list{$lib}}) {
            $lib64 = $check_list{$lib}{$lib32};
            print $ofh ("    32 : ");
            &print_lib_info($lib32);
            print $ofh ("    64 : ");
            &print_lib_info($lib64);
        }
    }
    
    print $ofh ("\n");
}

sub print_lib_info {
    my ($lib) = @_;
    my $msg = "";

    if (-e $lib) {
        $msg = `ls -al $lib | awk \'{print \$9 \$10 \$11}\'`;
        chomp($msg);
        print $ofh ("$msg\n");
    }
    else {
         print $ofh ("Could not find $lib\n");
    }
}

sub get_glibc_info {
    my $msg = "";

    print $ofh ("GLIBC :\n");
    print $ofh ("    32 : ");
    if (-e "/lib/libc.so.6") {
        $msg = `/lib/libc.so.6 | grep \"release version\"`;
        chomp($msg);
        print $ofh ("$msg\n");
    }
    else {
        print $ofh ("Could not find /lib/libc.so.6\n");
    }

    print $ofh ("    64 : ");
    if (-e "/lib64/libc.so.6") {
        $msg = `/lib64/libc.so.6 | grep \"release version\"`;
        chomp($msg);
        print $ofh ("$msg\n");
    }
    else {
        print $ofh ("Could not fine /lib64/libc.so.6\n");
    }
}

sub get_env_var_info {

    print $ofh ("##### Environemnt Variable Information #####\n");
    &get_env_var_with_avery;
    &get_license_env_var;
    &get_release_env_var;
    print $ofh ("\n");
}

sub get_env_var_with_avery {

    print $ofh ("Environment variables that contains AVERY : ");
    foreach (sort keys %ENV) {
        next if ( $_ !~ /AVERY/);
        print $ofh ("$_ ");
    }
    print $ofh ("\n");
}

sub get_license_env_var {
    my $lm_license_file = "";
    my $avery_license_file = "";   

    $lm_license_file = $ENV{'LM_LICENSE_FILE'};
    print $ofh ("LM_LICENSE_FILE : ");
    if (!defined $lm_license_file) {
        print $ofh ("Not found\n");
    }
    else {
        print $ofh ("$lm_license_file\n");
    }

    $avery_license_file = $ENV{'AVERYDES_LICENSE_FILE'};
    print $ofh ("AVERYDES_LICENSE_FILE : ");
    if (!defined $avery_license_file) {
        print $ofh ("Not found\n");
    }
    else {
        print $ofh ("$avery_license_file\n");
    }
}

sub get_release_env_var {
    my $avery_pli = "";
    my $avery_home = ""; 
   
    $avery_home = $ENV{'AVERY_HOME'};
    print $ofh ("AVERY_HOME : ");
    if (!defined $avery_home) {
        print $ofh ("Not found\n");
    }
    else {
        print $ofh ("$avery_home\n");
        $avery_dir = $avery_home;
    }
    
    $avery_pli = $ENV{'AVERY_PLI'};
    print $ofh ("AVERY_PLI : ");
    if (!defined $avery_pli) {
        print $ofh ("Not found\n");
    }
    else {
        print $ofh ("$avery_pli\n");
        $avery_dir = $avery_pli;
    }

    if (!defined $avery_dir) {
        print $ofh ("AVERY_PLI/AVERY_HOME not found\n");
        close($ofh);
        die ("AVERY_PLI/AVERY_HOME not found\n") if ($avery_dir eq "");
    }
}

sub get_lm_bin_info {
    my @binary = ("lmver", "lmstat");
    my $command = "";
    my $path = "";
    my $version = "";

    print $ofh ("##### FLEXLM Binary Information #####\n");
    foreach my $bin (@binary) {
        $command = "which ".$bin;
        $path = `$command`;
        chomp($path);
    
        if ($path =~ /no $bin in/) {
            print $ofh ("$bin : Not found\n");
            close($ofh);
            die ("Could not found $bin\n");
        }
        
        $version = &get_lm_bin_ver($path);
        print $ofh ("$bin\n");
        print $ofh ("    path : $path\n");
        print $ofh ("    version : $version\n");
    }
    print $ofh ("\n");
}

sub get_lm_bin_ver {
    my ($path) = @_;
    my $command = "lmver ".$path;
    my @msg = `$command`;

    foreach my $msg (@msg) {
        chomp($msg);
        next if ($msg !~ /FlexNet Licensing (v.*) build/);
        return $1;
    }
}

sub get_server_info {
    my $server_ver = "";
    my @msg = `lmstat`;

    print $ofh ("##### License Server Information #####\n");
    foreach my $line (@msg) {
        chomp($line);
        $line =~ s/^\s+//; # Trim left whitespace

        if ($line =~ /.*license server UP.*(v.*)/) {
            $server_ver = $1;
        }
        next if ($server_ver eq "");

        if ($line =~ /(.*): UP (v.*)/) {
            if ($1 eq "AVERYDES") {
                print $ofh ("Server : $server_ver\n");
                print $ofh ("AVERYDES : $2\n\n");
                return;
            }
            $server_ver = "";
        }
    }
    print $ofh ("No active Avery license server\n\n");
}

sub get_flexlmrc_info {
    my $flexlmrc = "~/.flexlmrc";
    my $fh;
    my $row = "";

    print $ofh ("##### flexlm.rc Information #####\n");
    if(!(open ($fh, "<", "\$HOME/.flexlmrc"))) {
        print $ofh ("No such file\n");
        return;
    }

    while ($row = <$fh>) {
        chomp $row;
        print $ofh ("$row\n");
    }

    close($fh);
}

#----------main----------------------
&parse_arg(\@ARGV);
open($ofh, ">".$log_file);
&get_sys_info;
&get_lib_info;
&get_env_var_info;
&get_lm_bin_info;
&get_server_info;
&get_flexlmrc_info;
close($ofh);
exit 0;
